import java.applet.*;
import java.awt.*;
import java.awt.event.*;

public class Mandelbrot extends Applet implements ActionListener, MouseListener, AdjustmentListener
{
	//alle benodigde componenten en variabelen declareren
	TextField xcorget, ycorget, schaalget, maxiterget;
	Scrollbar roodtempget, groentempget, blauwtempget;
	Button modeget, reset;
	Panel besturing, invoer, aanpassing;
	Label com1, com2, com3, com4, com5, com6, com7;
	double xcor, ycor, schaal, muisx, muisy, roodtemp, groentemp, blauwtemp;
	int maxiter, hoogte;
	byte mode;
	
	public void init()
	{
		//waarde van mode uit html-file halen, de rest kan tijdens de aanroep van de vakken
		mode = Byte.parseByte(this.getParameter("mode"));
		System.out.println(""+mode);
		hoogte = 400; //tijdelijk hoogte definieren
		//knoppenpaneel toevoegen, en dat paneel verder definieren
		//eerst layoutmanager van de hele applet uitzetten
		this.setLayout(null);
		//dan een rechter paneel declareren, die daarna in tween wordt opgedeeld
		besturing = new Panel();
		besturing.setBounds(hoogte+5,0,200,hoogte);
		this.add(besturing);
		besturing.setLayout(new GridLayout(2,1,1,1));
		invoer = new Panel();
		invoer.setBounds(0,0,200,200);
		invoer.setLayout(new GridLayout(8,1,1,1));
		besturing.add(invoer);
		aanpassing = new Panel();
		aanpassing.setBounds(0,0,200,200);
		aanpassing.setLayout(new GridLayout(8,1,1,1));
		besturing.add(aanpassing);
		
		//invoerbakken + commentaar definieren
		com1 = new Label("X-Cordinaat v/h midden v/h figuur");
		com2 = new Label("Y-Cordinaat v/h midden v/h figuur");
		com3 = new Label("De schaalfactor van het figuur");
		com4 = new Label("Max aantal berekeningen per pixel");
		xcorget = new TextField(this.getParameter("xcor"));
		ycorget = new TextField(this.getParameter("ycor"));
		schaalget = new TextField(this.getParameter("schaal"));
		maxiterget = new TextField(this.getParameter("maxiter"));
		invoer.add(com1);
		invoer.add(xcorget);
		invoer.add(com2);
		invoer.add(ycorget);
		invoer.add(com3);
		invoer.add(schaalget);
		invoer.add(com4);
		invoer.add(maxiterget);
		xcorget.addActionListener(this);
		ycorget.addActionListener(this);
		schaalget.addActionListener(this);
		maxiterget.addActionListener(this);
		
		//scrollbars + knoppen + commentaar definieren
		com5 = new Label("Rode kleurtemperatuur");
		com6 = new Label("Groene kleurtemperatuur");
		com7 = new Label("Blauwe kleurtemperatuur");
		roodtempget = new Scrollbar(0, 1, 1, 1, 255);
		groentempget = new Scrollbar(0, 1, 1, 1, 255);
		blauwtempget = new Scrollbar(0, 1, 1, 1, 255);
		aanpassing.add(com5);
		aanpassing.add(roodtempget);
		aanpassing.add(com6);
		aanpassing.add(groentempget);
		aanpassing.add(com7);
		aanpassing.add(blauwtempget);
		roodtempget.addAdjustmentListener(this);
		groentempget.addAdjustmentListener(this);
		blauwtempget.addAdjustmentListener(this);
		modeget = new Button("Switch mode");
		aanpassing.add(modeget);
		modeget.addActionListener(this);
		reset = new Button("Reset");
		aanpassing.add(reset);
		reset.addActionListener(this);
		//naar muis laten luisteren
		this.addMouseListener(this);
	}
	
	public void paint(Graphics g)
	{
		double mandelbrotgetal;
		int t1, t2; double x, y;
		xcor = Double.parseDouble(xcorget.getText());
		ycor = Double.parseDouble(ycorget.getText());
		schaal = Double.parseDouble(schaalget.getText());
		maxiter = Integer.parseInt(maxiterget.getText());
		roodtemp = roodtempget.getValue();
		groentemp = groentempget.getValue();
		blauwtemp = blauwtempget.getValue();
		
		for(t1=0; t1<hoogte; t1++)
		{
			for(t2=0; t2<hoogte; t2++)
			{
				x = pixel(schaal, hoogte, t1, xcor);
				y = pixel(schaal, hoogte, t2, ycor);
				mandelbrotgetal = mandel(x, y, maxiter);
				if(mandelbrotgetal%2==0)
				{
					g.setColor(kleur(mandelbrotgetal, mode));//new Color(255,255,255));
					g.drawLine(t1, t2, t1, t2);
				}
				else
				{
					g.setColor(kleur(mandelbrotgetal, mode));//new Color(0, 0, 0));
					g.drawLine(t1, t2, t1+1, t2+1);
				}
			}
		}
	}
	
	public Color kleur(double mandel, byte mode)
	{
		double rood, groen, blauw;
		if(mode==1)
		{
			rood=(((mandel%maxiter)*((255-roodtemp)/maxiter)));
			groen=(((mandel%maxiter)*((255-groentemp)/maxiter)));
			blauw=(((mandel%maxiter)*((255-blauwtemp)/maxiter)));
		}
		else if(mode==2)
		{
			rood=255-(((mandel%maxiter)*((255-roodtemp)/maxiter)));
			groen=255-(((mandel%maxiter)*((255-groentemp)/maxiter)));
			blauw=255-(((mandel%maxiter)*((255-blauwtemp)/maxiter)));
		}
		else
			rood=groen=blauw=0;
		return new Color((int)rood, (int)groen, (int)blauw);
	}
	
	public double mandel(double x, double y, int maxiter)
	{
		//uitrekenen mandelbrotgetal met f(a,b)=(a*a-b*b+x, 2*a*b+y), met a,b in begin 0,0
		int t; double ao, bo, a, b;
		a=b=0; ao=a; bo=b;
		for (t=1; t<maxiter; t++)
		{
			a = (ao*ao)-(bo*bo)+x;
			b = (2*ao*bo)+y;
			ao=a;
			bo=b;
			if(Math.sqrt(a*a + b*b)>4)
			{
				return t;
			}
		}
		return maxiter;
	}
	
	public double pixel(double schaal, double hoogte, double x, double cor)
	{
		//het maakt niet uit aan welke variabele de uitkomst van de pixelomrekening wordt
		//toegekend, aangezien hij in deze methode toch alleen lokaal wordt gebruikt
		//een extra variabele vraagt dus alleen extra systeembronnen
		x = (x*schaal - (hoogte/2)*schaal) + cor;
		return x;
	}
	
	//dit om de invoerbakken te laten reageren
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource()==modeget)
		{
			if(mode==1)
				mode = 2;
			else if(mode==2)
				mode = 1;
		}
		else if(e.getSource()==reset)
		{
			xcorget.setText("0");
			ycorget.setText("0");
			schaalget.setText("0.01");
			maxiterget.setText("100");
		}
		this.repaint();
	}
	
	//deze methoden om naar de muis te laten luisteren
	public void mousePressed(MouseEvent e)
	{
		
	}
	public void mouseReleased(MouseEvent e)
	{
		
	}
	public void mouseClicked(MouseEvent e)
	{
		muisx = e.getX();
		muisy = e.getY();
		//er mag alleen in of uit gezoomed worden als binnen de grenzen van het figuur wordt geklikt
		if(!(muisx>400 || muisy>400))
		{
			xcorget.setText(""+pixel(schaal, hoogte, muisx, xcor));
			ycorget.setText(""+pixel(schaal, hoogte, muisy, ycor));
			if(e.getButton()==e.BUTTON1)
				schaalget.setText(""+(schaal*0.5));
			if(e.getButton()==e.BUTTON3)
				schaalget.setText(""+(schaal*2));
			this.repaint();
		}
	}
	public void mouseEntered(MouseEvent e)
	{
		
	}
	public void mouseExited(MouseEvent e)
	{
		
	}
	//deze methode om naar de scrollbars te luisteren
	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		this.repaint();
	}
}